﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using RoundLineCode;
using Microsoft.Xna.Framework.Content;

namespace PlanetaryDomination
{
    public class BattleManager
    {// verwaltet die sektoren und zeigt sie an
        public List<BattleSector> Battlefield;
        private GraphicsManager Graphics;
        InputManager Input;
        FontManager Fonts;
        SoundManager Sound;
        GameData Gamedata;
        public ViewManager Views;
        int NumberofSectors, ActSelSector, SectorHeight, SectorZoomHeight, MouseSecX, MouseSecY, MouseSector, LastSelSector, LastHighlightSector, UpperLeftX, UpperLeftY, LowerRightX, LowerRightY, MoveStart, MoveEnd;
        Vector2 PosVector;
        float MouseScrX, MouseScrY, PixelHeight, SectorLength, ZoomSectorLength, HalfSectorLength;
        public bool Movemode, Attackmode;
        public Unit UnittoMove;
        public int ActiveUnit;

        RoundLineManager roundLineManager;
        RoundLine MoveLine;
        Matrix viewMatrix,projMatrix;
        float cameraX = 400.0f, cameraY = -280.0f,cameraZoom = 300;
        int roundLineTechniqueIndex = 0;
        string[] roundLineTechniqueNames;

        public BattleManager(GraphicsManager graphicman, InputManager inputman, FontManager fontman, SoundManager soundman, GameData gamedat, ContentManager content)
        {// constructor
            Battlefield = new List<BattleSector>();
            PosVector = new Vector2();
            Graphics = graphicman;
            Fonts = fontman;
            Input = inputman;
            Sound = soundman;
            Gamedata = gamedat;
            NumberofSectors = 18;
            PixelHeight = Graphics.ScrWidth * 0.5f;
            Movemode = Attackmode = false;
            roundLineManager = new RoundLineManager();
            roundLineManager.Init(Graphics.Device, content);
            roundLineTechniqueNames = roundLineManager.TechniqueNames;
            MoveLine = new RoundLine(10.0f, 10.0f, 790.0f, -370.0f);
            Create2DProjectionMatrix();

            SectorZoomHeight = SectorHeight = 3;
            SectorLength = PixelHeight / (float)SectorHeight;
            ZoomSectorLength = PixelHeight / (float)SectorZoomHeight; // die kantenlänge eines quadratischen sectors in pixel ausrechnen
            SectorLength = PixelHeight / (float)SectorHeight;
            HalfSectorLength = ZoomSectorLength * 0.5f; // und halbieren
            ActSelSector = LastSelSector = LastHighlightSector;  // auswahl auf heimatsektor des spielers setzen
            UpperLeftX = UpperLeftY = 0;    // initial gesamtes spielfeld anzeigen
            LowerRightX = SectorHeight * 2 - 1;
            LowerRightY = SectorHeight - 1;

            int i, column = 0, line = 0;
            for (i = 0; i < NumberofSectors; i++)
            {
                Battlefield.Add(new BattleSector(column, line, i));
                if (++column == 6)
                {
                    column = 0;
                    ++line;
                }
            }
        }
        public void Update()
        {// maus usw auswerten
            viewMatrix = Matrix.CreateTranslation(-cameraX, -cameraY, 0) * Matrix.CreateScale(1.0f / cameraZoom, 1.0f / cameraZoom, 1.0f);
            MouseScrX = Input.GetMousePosX(); // update der koordinaten für die maus
            MouseScrY = Input.GetMousePosY();
            if ((MouseScrY < PixelHeight) && (MouseScrY > 0.0f) && (MouseScrX > 0.0f) && (MouseScrX < PixelHeight*2.0f))
            {// die maus ist im spielfeld also position usw als eingabe werten
                MouseSecX = (int)Math.Floor((float)SectorZoomHeight * MouseScrX / PixelHeight) + UpperLeftX;
                MouseSecY = (int)Math.Floor((float)SectorZoomHeight * MouseScrY / PixelHeight) + UpperLeftY;
                MouseSector = MouseSecY * SectorHeight * 2 + MouseSecX; // den index des sektors über dem sich maus befindet berechnen
                if (LastHighlightSector != MouseSector)
                {// ein anderer sector wird gehighlighted also sound spielen
                    LastHighlightSector = MouseSector;
                    Sound.PlaySound(2);
                }
                if (Movemode)
                {// bewegungsmodus aktiv
                    MoveEnd = MouseSector;  // wenn wir im Bewegungsmodus sind ziel fürs bewegen setzen
                    if (Input.IsFirstPressed(1))    // wenn im bewegungsmodus bei rmt bewegungsmodus abbrechen
                        Movemode = false;
                }
                if (Input.IsFirstPressed(0))
                {// maus wurde gedrückt also Sound spielen wenn neuer sektor ausgewählt
                    if (Movemode)   //# hier die anderen prüfungen machen
                    {// im movemode einheit mit linksklick bewegen falls möglich
                        if (Battlefield[LastHighlightSector].Moveable)
                        {// prüfen ob es geht
                            MoveUnit(Battlefield[ActSelSector], Battlefield[LastHighlightSector], UnittoMove);
                        }
                        else
                        {//# ansonsten ton spielen
                        }
                        Movemode = false;   // in jedem fall movemode beenden
                    }
                    else if (Attackmode)
                    {
                        if (Battlefield[LastHighlightSector].Attackable)
                        {// prüfen ob es geht
                            AttackSector(Battlefield[ActSelSector], Battlefield[LastHighlightSector], UnittoMove);
                        }
                        else
                        {//# ansonsten ton spielen
                        }
                        Attackmode = false;   // in jedem fall attackmode beenden
                    }
                    else
                    {// im normalen modus sektor auswählen
                        ActSelSector = MouseSector;
                        if (LastSelSector != ActSelSector)
                        {// neuer Sector ausgewählt also Sound spielen
                            LastSelSector = ActSelSector;
                            Sound.PlaySound(3);
                        }
                    }
                }
            }
        }
        public void Draw()
        {// zeichnet die sektoren in einer schleife
            BattleSector tempsec;
            for (int i = 0; i < NumberofSectors; i++)
            {
                tempsec = ((BattleSector)(Battlefield[i]));
                PosVector.X = (tempsec.X - UpperLeftX) * ZoomSectorLength + HalfSectorLength;
                PosVector.Y = (tempsec.Y - UpperLeftY) * ZoomSectorLength + HalfSectorLength;
                if (Movemode)
                {
                    if(tempsec.Moveable)
                        Graphics.DrawImagePart((int)(PosVector.X - HalfSectorLength), (int)(PosVector.Y - HalfSectorLength), (int)ZoomSectorLength, (int)ZoomSectorLength, 0, 0, 320, 320, 8, 0.0f);
                    else
                        Graphics.DrawImagePart((int)(PosVector.X - HalfSectorLength), (int)(PosVector.Y - HalfSectorLength), (int)ZoomSectorLength, (int)ZoomSectorLength, 0, 0, 320, 320, 7, 0.0f);
                }
                if (ActSelSector == tempsec.Number)
                {// der selektierte sektor
                    Graphics.DrawImagePart((int)(PosVector.X - HalfSectorLength), (int)(PosVector.Y - HalfSectorLength), (int)ZoomSectorLength, (int)ZoomSectorLength, 0, 0, 320, 320, 4, 0.0f);
                }
                else if (LastHighlightSector == tempsec.Number)
                {// der gehighlightete sector
                    Graphics.DrawImagePart((int)(PosVector.X - HalfSectorLength), (int)(PosVector.Y - HalfSectorLength), (int)ZoomSectorLength, (int)ZoomSectorLength, 0, 0, 320, 320, 2, 0.0f);
                }
                else
                {// ein normaler sektor also nur den rand zeichnen
                    Graphics.DrawImagePart((int)(PosVector.X - HalfSectorLength), (int)(PosVector.Y - HalfSectorLength), (int)ZoomSectorLength, (int)ZoomSectorLength, 0, 0, 320, 320, 3, 0.0f);
                }
                if (tempsec.Unit != null)
                {
                    Graphics.DrawText(String.Format("{0}", tempsec.Unit.Name), PosVector, Gamedata.Players[tempsec.Unit.Owner].Playercolor, 5);// hier die sektordetails + werte zeichnen (oder von sektoren selbst zeichnen lassen)
                }
            }
            if (Movemode)
            {// einheit wird bewegt also farbige linie zwischen start und endsector zeichnen
                Graphics.SpriteBatch.End(); // alles bisherige zeichnen damit die linie drüber gemalt wird
                Matrix viewProjMatrix = viewMatrix * projMatrix;
                roundLineManager.NumLinesDrawn = 0;
                float lineRadius = 5;
                string curTechniqueName = roundLineTechniqueNames[roundLineTechniqueIndex];
                MoveLine.P0 = GetSectorCenter(ActSelSector);
                MoveLine.P1 = GetSectorCenter(LastHighlightSector);
                if (Battlefield[LastHighlightSector].Moveable)
                    roundLineManager.Draw(MoveLine, lineRadius, Color.Green, viewProjMatrix, 0.0f, curTechniqueName);
                else
                    roundLineManager.Draw(MoveLine, lineRadius, Color.Red, viewProjMatrix, 0.0f, curTechniqueName);
                Graphics.SpriteBatch.Begin();   // und 2d zeichnen neu starten für texte usw
                Graphics.DrawText("Move to", new Vector2(1280.0f * (float)(Battlefield[LastHighlightSector].X - UpperLeftX) / (float)(LowerRightX - UpperLeftX + 1) + 320.0f / (float)(LowerRightX - UpperLeftX + 1), 640.0f * (float)(Battlefield[LastHighlightSector].Y - UpperLeftY) / (float)(LowerRightY - UpperLeftY + 1) + 320.0f / (float)(LowerRightY - UpperLeftY + 1)), Color.White, 4); ;
            }
        }
        public BattleSector GetActSector()
        {
            return Battlefield[ActSelSector];
        }
        public void SetMoveMode(int startsect, Unit moveunit)
        {// eine linie zwischen startsect und highlightsect zeichnen und unit bei click in neuen sector einfügen und eta setzen
            Movemode = true;
            UnittoMove = moveunit;
            MoveStart = startsect;
            MoveEnd = startsect;
            BattleSector tempsect;
            for (int i = 0; i < Battlefield.Count; i++)
            {
                tempsect = Battlefield[i];
                if ((tempsect.Unit == null) && (GetSectorDistance(tempsect, Battlefield[startsect]) <= moveunit.PlanetMoveRange))
                {// sektor unbewohnt also prüfen ob in reichweite und ob überhaupt platz ist im zielsector
                        tempsect.Moveable = true;
                }
                else
                {// da ist schon was oder zu weit weg
                    tempsect.Moveable = false;
                }
            }
        }
        public void Create2DProjectionMatrix()
        {// Create a simple 2D projection matrix (matrix ignores Z and just squishes X or Y to balance the upcoming viewport stretch)
            float projScaleX,projScaleY,width = Graphics.Device.Viewport.Width,height = Graphics.Device.Viewport.Height;
            if (width > height)
            {// Wide window
                projScaleX = height / width;
                projScaleY = 1.0f;
            }
            else
            {// Tall window
                projScaleX = 1.0f;
                projScaleY = width / height;
            }
            projMatrix = Matrix.CreateScale(projScaleX, projScaleY, 0.0f);
            projMatrix.M43 = 0.5f;
        }
        public Vector2 GetSectorCenter(int sector)
        {// liefert die transformierten bildschirmkoordinaten des sektorzentrums
            Vector2 result;
            result.X = 800.0f * (float)(Battlefield[sector].X - UpperLeftX) / (float)(LowerRightX - UpperLeftX + 1) + 400.0f / (float)(LowerRightX - UpperLeftX + 1);
            result.Y = -400.0f * (float)(Battlefield[sector].Y - UpperLeftY) / (float)(LowerRightY - UpperLeftY + 1) - 100.0f / (float)(LowerRightY - UpperLeftY + 1);
            return result;
        }
        public int GetSectorDistance(BattleSector sec1, BattleSector sec2)
        {// bestimmt den Abstand zwischen den beiden sektoren aber als dx+dy also keine diagonale
            return Math.Max(sec1.X, sec2.X) - Math.Min(sec1.X, sec2.X) + Math.Max(sec1.Y, sec2.Y) - Math.Min(sec1.Y, sec2.Y);
        }
        public void MoveUnit(BattleSector fromsec, BattleSector tosec, Unit unit)
        {// bewegt die einheit in den gewünschten sektor welche sofort da ankommt
            tosec.Unit = unit;
            unit.Battlesec = tosec.Number;
            fromsec.Unit = null;
            unit.TurnMoved = true;
        }
        public bool SectorAttackable()
        {// bestimmt ob mindestens 1 nachbarsektor angreifbar ist (feind sitzt drin bzw hat einheit dort)
            bool attackable = false;
            for (int i = 0; i < Battlefield.Count; i++)
            {
                if ((i != ActSelSector) && (Battlefield[i].Unit.MinFireRange <= GetSectorDistance(Battlefield[i], Battlefield[ActSelSector])) && (Battlefield[i].Unit.MaxFireRange >= GetSectorDistance(Battlefield[i], Battlefield[ActSelSector])))
                {// nicht der gerade ausgewählte sector und in der feuerreichweite
                    Battlefield[i].Attackable = true;
                    attackable = true;
                }
                else
                {
                    Battlefield[i].Attackable = false;
                }
            }
            return attackable;
        }
        public void SetAttackMode(int startsect, Unit moveunit)
        {// eine linie zwischen startsect und highlightsect zeichnen und unit bei click in neuen sector einfügen und eta setzen
            Attackmode = true;
            MoveStart = startsect;
            MoveEnd = startsect;
            UnittoMove = moveunit;
        }
        public void AttackSector(BattleSector fromsec, BattleSector tosec, Unit unit)
        {// greift einen sector an
//            fromsec.Unit.AttackUnit(tosec.Unit);
        }
    }
}
