﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.Reflection;
using PlanetaryDomination;

namespace Editor
{
    public partial class PDEditor : Form
    {
        private int ActUnit;
        private UnitManager Units;

        public PDEditor()
        {
            InitializeComponent();
            ActUnit = 0;
            Units = new UnitManager();
//            Units.LoadTechTree("\\TechQueue.dat");
            UpdateScreenDatafromUnit();
        }
        private void FirstUnit_Click(object sender, EventArgs e)
        {
            ActUnit = 0;
            UpdateScreenDatafromUnit();
        }
        private void PrevUnit_Click(object sender, EventArgs e)
        {
            if (ActUnit > 0)
            {
                --ActUnit;
                UpdateScreenDatafromUnit();
            }
        }
        private void NextUnit_Click(object sender, EventArgs e)
        {
            if (ActUnit < Units.Technologies.Count)
            {
                ++ActUnit;
                UpdateScreenDatafromUnit();
            }
        }
        private void LastUnit_Click(object sender, EventArgs e)
        {
            ActUnit = Units.Technologies.Count - 1;
            UpdateScreenDatafromUnit();
        }
        private void AddUnit_Click(object sender, EventArgs e)
        {
            Units.Technologies.Add(new Unit());
            ActUnit = Units.Technologies.Count -1;
            UpdateScreenDatafromUnit();
        }
        private void DelUnit_Click(object sender, EventArgs e)
        {
            if (Units.Technologies.Count > 0)
            {
                Units.Technologies.RemoveAt(ActUnit);
                ActUnit = 0;
                UpdateScreenDatafromUnit();
            }
        }
        private void UpdateScreenDatafromUnit()
        {// nimmt die daten aus dem array und schreibt sie in die bildschirmvariablen
            if ((Units.Technologies.Count > 0) && (ActUnit >= 0) && (ActUnit < Units.Technologies.Count))
            {// nur wenn gültig anzeigen
                MaxUnit.Text = String.Format("/ {0}", Units.Technologies.Count);
                Position.Text = String.Format("{0}", ActUnit + 1);
                Unit tempunit = Units.Technologies[ActUnit];
                UnitName.Text = tempunit.Name;
                UnitType.SelectedIndex = tempunit.Type;
                UnitAttack.Text = String.Format("{0}", tempunit.Attack);
                UnitDefence.Text = String.Format("{0}", tempunit.Defence);
                UnitHP.Text = String.Format("{0}", tempunit.Maxhitpoints);
                UnitProdCost.Text = String.Format("{0}", tempunit.ProductionCost);
                UnitMinCost.Text = String.Format("{0}", tempunit.MinCost);
                UnitOrgCost.Text = String.Format("{0}", tempunit.OrgCost);
                UnitEnergyCost.Text = String.Format("{0}", tempunit.EnergyCost);
                MinFireRange.Text = String.Format("{0}", tempunit.MinFireRange);
                MaxFireRange.Text = String.Format("{0}", tempunit.MaxFireRange);
                BattleMoveRange.Text = String.Format("{0}", tempunit.BattleMoveRange);
                PlanetMoveRange.Text = String.Format("{0}", tempunit.PlanetMoveRange);
                UnitSpeed.Text = String.Format("{0}", tempunit.Speed);
                MinFactoryLevel.Text = String.Format("{0}", tempunit.MinFactoryLevel);
                MinTechLevel.Text = String.Format("{0}", tempunit.MinTechLevel);
            }
        }
        private void UpdateUnitDatafromDataScreen()
        {// nimmt die bildschirmvariablen und schreibt sie in die daten im array
            Unit tempunit = Units.Technologies[ActUnit];
            float tempfloat;
            bool error = false;
            StringBuilder errorstring = new StringBuilder(256);
            errorstring.Append("Errors occured during procession of the follwing values:\n");
            if((UnitName.Text.Length > 20) || (UnitName.Text.Length < 3))
            {
                error = true;
                errorstring.Append("Name\n");
            }
            if ((false == float.TryParse(UnitAttack.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 9.0f))
            {
                error = true;
                errorstring.Append("Attack\n");
            }
            if ((false == float.TryParse(UnitDefence.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 9.0f))
            {
                error = true;
                errorstring.Append("Defence\n");
            }
            if ((false == float.TryParse(UnitHP.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 99.0f))
            {
                error = true;
                errorstring.Append("Hitpoints\n");
            }
            if ((false == float.TryParse(UnitProdCost.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 90.0f))
            {
                error = true;
                errorstring.Append("Productioncost\n");
            }
            if ((false == float.TryParse(UnitMinCost.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 90.0f))
            {
                error = true;
                errorstring.Append("Mineralcost\n");
            }
            if ((false == float.TryParse(UnitOrgCost.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 90.0f))
            {
                error = true;
                errorstring.Append("OrganicsCost\n");
            }
            if ((false == float.TryParse(UnitEnergyCost.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 90.0f))
            {
                error = true;
                errorstring.Append("EnergyCost\n");
            }
            if ((false == float.TryParse(MinFireRange.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 9.0f))
            {
                error = true;
                errorstring.Append("MinFireRange\n");
            }
            if ((false == float.TryParse(MaxFireRange.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 9.0f))
            {
                error = true;
                errorstring.Append("MaxFireRange\n");
            }
            if ((false == float.TryParse(BattleMoveRange.Text, out tempfloat)) || (tempfloat < 0.0f) || (tempfloat > 9.0f))
            {
                error = true;
                errorstring.Append("BattleMoveRange\n");
            }
            if ((false == float.TryParse(PlanetMoveRange.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 5.0f))
            {
                error = true;
                errorstring.Append("PlanetMoveRange\n");
            }
            if ((false == float.TryParse(UnitSpeed.Text, out tempfloat)) || (tempfloat < 0.0f) || (tempfloat > 10.0f))
            {
                error = true;
                errorstring.Append("UnitSpeed\n");
            }
            if ((false == float.TryParse(MinFactoryLevel.Text, out tempfloat)) || (tempfloat < 1.0f) || (tempfloat > 5.0f))
            {
                error = true;
                errorstring.Append("MinFactoryLevel\n");
            }
            if ((false == float.TryParse(MinTechLevel.Text, out tempfloat)) || (tempfloat < 0.0f) || (tempfloat > 100000.0f))
            {
                error = true;
                errorstring.Append("MinTechLevel\n");
            }
            if (false == error)
            {// alles hat gestimmt also jetzt werte übernehmen
                tempunit.Name = UnitName.Text;
                tempunit.Type = UnitType.SelectedIndex;
                tempunit.Attack = float.Parse(UnitAttack.Text);
                tempunit.Defence = float.Parse(UnitDefence.Text);
                tempunit.Maxhitpoints = float.Parse(UnitHP.Text);
                tempunit.ProductionCost = int.Parse(UnitProdCost.Text);
                tempunit.MinCost = int.Parse(UnitMinCost.Text);
                tempunit.OrgCost = int.Parse(UnitOrgCost.Text);
                tempunit.EnergyCost = int.Parse(UnitEnergyCost.Text);
                tempunit.MinFireRange = int.Parse(MinFireRange.Text);
                tempunit.MaxFireRange = int.Parse(MaxFireRange.Text);
                tempunit.BattleMoveRange = int.Parse(BattleMoveRange.Text);
                tempunit.PlanetMoveRange = int.Parse(PlanetMoveRange.Text);
                tempunit.Speed = float.Parse(UnitSpeed.Text);
                tempunit.MinFactoryLevel = int.Parse(MinFactoryLevel.Text);
                tempunit.MinTechLevel = int.Parse(MinTechLevel.Text);
            }
            else
            {// etwas stimmt nicht also meldung machen
                errorstring.Append("Please correct the errors and proceed!");
                MessageBox.Show(errorstring.ToString());
            }
        }
        private void SaveUnits_Click(object sender, EventArgs e)
        {// schreibt die daten in die datei (überschreiben ohne nachfrage), das wird nicht als methode im unitmanager gemacht um missbrauch zu vermeiden und da die daten im spiel nicht geändert werden können
            Units.SaveTechTree("\\TechQueue.dat");
        }
        private void SaveUnit_Click(object sender, EventArgs e)
        {// merkt sich die einstellungen für diese einheit
            UpdateUnitDatafromDataScreen();
        }
    }
}
