﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace PlanetaryDomination
{
    public class GraphicsManager
    {// verwaltet alle zeichenoperationen usw
        private TextureManager Textures;
        private FontManager Fonts;
        public GraphicsDeviceManager Graphics;
        public GraphicsDevice Device;

        public SpriteBatch SpriteBatch;
        public int ScrWidth, ScrHeight;    // auflösung des relevanten spielfeldes (der bildschirm/backbuffer kann später evtl andere auflösung (größer) haben was durch schwarz aufgefüllt wird)

        Texture2D TempTexture;
        Vector2 TempVectorPos, TempVectorOrig;
        Rectangle SourceRect, DestRect;

        public GraphicsManager(Game game)
        {
            ScrWidth = 1280;
            ScrHeight = 960;
            Graphics = new GraphicsDeviceManager(game);
            Graphics.PreferredBackBufferWidth = ScrWidth;
            Graphics.PreferredBackBufferHeight = ScrHeight;
            Graphics.IsFullScreen = false;
            Graphics.PreferredDepthStencilFormat = DepthFormat.Depth32;
            Graphics.PreferMultiSampling = true;
            Graphics.SynchronizeWithVerticalRetrace = true;
            Device = Graphics.GraphicsDevice;
        }
        public void Init(TextureManager textman, FontManager fontman)
        {
            Device = Graphics.GraphicsDevice;
            Textures = textman;
            Fonts = fontman;
            SpriteBatch = new SpriteBatch(Graphics.GraphicsDevice);// Create a new SpriteBatch, which can be used to draw textures.
            TempVectorPos = new Vector2();
            TempVectorOrig = new Vector2();
            SourceRect = new Rectangle();
            DestRect = new Rectangle();
        }
        public bool MouseinWindow(int mousex, int mousey)
        {// prüft ob sich die maus innerhalb des fensters befindet
            //# wie ist das hier mit vollbild und fenster? und ist 0,0 immer die obere ecke?
            //# das ganze noch auf die aktuelle auflösung beziehen
            return ((mousex >= 0) && (mousex <= ScrWidth) && (mousey >= 0) && (mousey <= ScrHeight));
        }
        public void DrawImage(float x, float y, int textureindex, float rotation, float scale)
        {// zeichnet eine textur aus der liste an die gewählte position
            TempTexture = Textures.GetTexture(textureindex);
            TempVectorOrig.X = TempTexture.Width / 2;
            TempVectorOrig.Y = TempTexture.Height / 2;
            TempVectorPos.X = x;
            TempVectorPos.Y = y;
            SpriteBatch.Draw(TempTexture, TempVectorPos, null, Color.White, rotation, TempVectorOrig, scale, SpriteEffects.None, 0.0f);
        }
        public void DrawImagePart(int destx, int desty, int destwidth, int destheight, int srcx, int srcy, int srcwidth, int srcheight, int textureindex, float rotation)
        {// zeichnet eine textur aus der liste an die gewählte position
            TempTexture = Textures.GetTexture(textureindex);
//            TempVectorOrig.X = TempTexture.Width / 2;   //# das hier für animationen anpassen damit die mitte der animation für roation genommen wird, einfach ausrechnen aus x+destwidth/2?
//            TempVectorOrig.Y = TempTexture.Height / 2;
            TempVectorOrig.X = 0.0f;   //# das hier für animationen anpassen damit die mitte der animation für roation genommen wird, einfach ausrechnen aus x+destwidth/2?
            TempVectorOrig.Y = 0.0f;
            SourceRect.X = srcx;
            SourceRect.Y = srcy;
            SourceRect.Width = srcwidth;
            SourceRect.Height = srcheight;
            DestRect.X = destx;
            DestRect.Y = desty;
            DestRect.Width = destwidth;
            DestRect.Height = destheight;
            SpriteBatch.Draw(TempTexture, DestRect, SourceRect, Color.White, rotation, TempVectorOrig, SpriteEffects.None, 0.0f);
        }
        public void DrawText(String drawtext, Vector2 upperleftcorner, Color color, int font)
        {// draws spezified text in the spezified rectangle and picks largest matching fontsize
            TempVectorPos.X = upperleftcorner.X;
            TempVectorPos.Y = upperleftcorner.Y;
            SpriteBatch.DrawString(Fonts.GetFont(font), drawtext, TempVectorPos, color);
        }
        public void ToggleFullscreen()
        {
            Graphics.ToggleFullScreen();
        }
        public void Clear(byte r, byte g, byte b)
        {// Clear the window
            Graphics.GraphicsDevice.Clear(new Color(r, g, b));
        }
        public void GetStringSize(String measurestring, Vector2 range, ref Vector2 finalsize, ref int fontsize)
        {// starts with the largest font and decreases size until the string matches in the given range
            Vector2 fontrange;
            fontrange.X = fontrange.Y = 0.0f;
            for (int i = Fonts.GetFontCount() - 1; i >= 0; i--)
            {// traverse backwards through the fonts (from large to small) and pick the first one which matches in the range
                fontrange = Fonts.GetFont(i).MeasureString(measurestring);
                if ((fontrange.X <= range.X) && (fontrange.Y <= range.Y))
                {
                    fontsize = i;
                    finalsize =  new Vector2(fontrange.X,fontrange.Y);
                    break;  // wenn erste passende schrift gefunden verlassen
                }
            }
        }
        public void GetStringSize(String measurestring, int fontsize, ref Vector2 finalsize)
        {// starts with the largest font and decreases size until the string matches in the given range
            Vector2 fontrange;
            fontrange.X = fontrange.Y = 0.0f;
            fontrange = Fonts.GetFont(fontsize).MeasureString(measurestring);
            finalsize = new Vector2(fontrange.X, fontrange.Y);
        }
    }
}
