using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace PlanetaryDomination
{
    public class PDGame : Microsoft.Xna.Framework.Game
    {// This is the main type for your game
        Random RandomNumberGenerator;
        InputManager Input;
        SoundManager Sound;
        GUIManager Gui;
        TextureManager Textures;
        FontManager Fonts;
        GraphicsManager Graphics;
        GameData Gamedata;
        bool GamePaused;

        public PDGame()
        {
            Graphics = new GraphicsManager((Game)this); // graphics device initialisieren
            Content.RootDirectory = "Content";
        }
        protected override void Initialize()
        {// Allows the game to perform any initialization it needs to before starting to run. This is where it can query for any required services and load any non-graphic related content.  Calling base.Initialize will enumerate through any components and initialize them as well.
            RandomNumberGenerator = new Random();
            Input = new InputManager();
            Input.Init();
            Sound = new SoundManager();
            Fonts = new FontManager(Content);
            Textures = new TextureManager(Content, Graphics);
            Graphics.Init(Textures, Fonts);
            Textures.Init();
            Gamedata = new GameData();
            Gamedata.Sectors = new SectorManager(Graphics, Input, Fonts, Sound, Gamedata, Content);
            Gui = new GUIManager(Sound, Input, Graphics,Fonts, Gamedata);
            GamePaused = false;
            ((Button)(Gui.Views.Views[0].Controls[4])).ButtFunc = Exit;
            ((MapView)(Gui.Views.Views[1])).Gamedata = Gamedata;
            ((CreateGameView)(Gui.Views.Views[2])).Gamedata = Gamedata;
            ((BattleView)(Gui.Views.Views[4])).BattleMan = new BattleManager(Graphics, Input, Fonts, Sound, Gamedata, Content);
            Gamedata.Sectors.Views = Gui.Views;
            base.Initialize();
        }
        protected override void LoadContent()
        {// LoadContent will be called once per game and is the place to load all of your content.
        }
        protected override void UnloadContent()
        {// UnloadContent will be called once per game and is the place to unload all content.
        }
        protected override void Update(GameTime gameTime)
        {// Allows the game to run logic such as updating the world, checking for collisions, gathering input, and playing audio.
            Input.Update(gameTime);
            if (Graphics.MouseinWindow(Input.GetMousePosX(), Input.GetMousePosY()))
            {
                if (Input.IsFirstPressed(Keys.Escape))  // bei spiel verlassen
                    Exit();
/*                if (Input.IsFirstPressed(Keys.Pause))   // pause taste pausiert spiel (//# viewunabhngig?) ist das in rundenstrategie ntig?
                    GamePaused = !GamePaused;*/
                if (Input.IsFirstPressed(Keys.F12))     // f12 schaltet zwischen vollbild und fenstermodus um
                    Graphics.ToggleFullscreen();
                if (Input.IsFirstPressed(Keys.F5))
                    Gamedata.SaveGame("\\Quicksave.sav"); //# das evtl noch in der richtigen view machen oder kann man das von berall?
                if (Input.IsFirstPressed(Keys.F6))
                    Gamedata.LoadGame("\\Quicksave.sav");
                Gui.Update();
                if (!GamePaused)
                {// wenn spieler nicht verloren hat und spiel nicht pausiert ist input verarbeiten und spiel update (bewegen, ki)
                    Gamedata.Update();
                }
            }
            Sound.Audio.Update();

            base.Update(gameTime);
        }
        protected override void Draw(GameTime gameTime)
        {// This is called when the game should draw itself.
            Graphics.Graphics.GraphicsDevice.Clear(Color.Black);
            Graphics.SpriteBatch.Begin();
            Gui.Draw();
            Graphics.SpriteBatch.End();
            base.Draw(gameTime);
        }
    }


    static class Program
    {
        static void Main(string[] args)
        {// The main entry point for the application.
            try
            {// falls was schief geht wird der grund in einer messagebox angezeigt
                using (PDGame game = new PDGame())
                {
                    game.Run();
                }
            }
            catch (Exception e)
            {
                System.Windows.Forms.MessageBox.Show(e.ToString());
            }
        }
    }
}
