﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace PlanetaryDomination
{
    public class ViewManager
    {// holds a list of all views, remembers the actual view and handles changes between views, assigns input operations to gui and scene
        public List<BaseView> Views; // enthält alle views
        int AktView,OldView;  // indiziert die aktuelle view im view-array
        public InputManager Input;
        public GraphicsManager Graphics;
        public SoundManager Sound;
        public GameData Gamedata;

        public ViewManager(InputManager inputman, GraphicsManager graphicman, GameData gamedat, SoundManager soundman)
        {// constructor
            Views = new List<BaseView>();
            AktView = OldView = 0;
            Input = inputman;
            Graphics = graphicman;
            Gamedata = gamedat;
            Sound = soundman;
        }
        public void Init()
        {// inits the view-array with all views
            Views.Add(new MenuView(Graphics, Input, Sound, this));  // #0 hauptmenü
            Views.Add(new MapView(Graphics, Input, Sound, this));   // #1 kartenansicht
            Views.Add(new CreateGameView(Graphics, Input, Sound, this));   // #2 neues spiel über einstellungen konfigurieren (farbe, größe, gegner)
            Views.Add(new CreditsView(Graphics, Input, Sound, this));   // #3 credits + software
            Views.Add(new BattleView(Graphics, Input, Sound, this));   // #4 kampfansicht
            Views.Add(new ResearchView(Graphics, Input, Sound, this));   // #5 forschungsansicht
            for (int i = 0; i < Views.Count; i++)
            {// schleife um alle views zu initialisieren
//                Views[i].InitView(Graphics);
            }
            ((MapView)(Views[1])).Sectors = Gamedata.Sectors;
        }
        public void Draw()
        {// rendert die aktuelle view
            Views[AktView].Draw();
        }
        public void SwitchView(int newview)
        {// switches the view
            if (newview != AktView)
            {// check if same view is set again
                OldView = AktView;
                AktView = newview;
            }
        }
        public void Update()
        {// delegate the task to check the input to the active view
            // first check for general view-independant input
            if (Input.IsPressed(Keys.F1))
            {// f1 main menu
                SwitchView(0);
            }
            if (Input.IsPressed(Keys.F2))
            {// f2 map view
                SwitchView(1);
            }
            Views[AktView].Update();
        }
    }
}
